import { createToken, createPlugin, memoize } from 'fusion-core';
import bodyParser from 'koa-bodyparser';
import { matchPath } from 'react-router';

var browserPlugin = null;

var HttpRouterToken = createToken('HttpRouter');
var HttpHandlersToken = createToken('HttpHandlersToken');
var BodyParserOptionsToken = createToken('BodyParserOptionsToken');

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

/* eslint-env node */
function getHttpHandler(currentPath, method, paths, handlers) {
  var _ref = matchPath(currentPath, {
    path: paths
  }) || {},
      path = _ref.path,
      _ref$params = _ref.params,
      params = _ref$params === void 0 ? {} : _ref$params,
      isExact = _ref.isExact;

  var handler = handlers[path] && handlers[path][method];

  if (path && handler && isExact) {
    if (typeof handler !== 'function') {
      throw new Error("Missing/incorrect handler registered to " + method + " of " + path + ".");
    }

    return [handler, params];
  }

  return [];
}
var plugin = createPlugin({
  deps: {
    bodyParserOptions: BodyParserOptionsToken.optional,
    handlers: HttpHandlersToken
  },
  provides: function provides(_ref2) {
    var handlers = _ref2.handlers,
        bodyParserOptions = _ref2.bodyParserOptions;

    if (!handlers || _typeof(handlers) !== 'object') {
      throw new Error('Missing/incorrect handlers registered to HttpHandlersToken');
    }

    var parseBody = bodyParser(bodyParserOptions);
    var paths = Object.keys(handlers);
    var from = memoize(function (ctx) {
      return new Promise(function ($return, $error) {
        var path, method, query, _getHttpHandler, handler, params;

        path = ctx.path, method = ctx.method, query = ctx.query;
        _getHttpHandler = getHttpHandler(path, method, paths, handlers), handler = _getHttpHandler[0], params = _getHttpHandler[1];

        if (typeof handler !== 'function') {
          return $return(null);
        }

        return Promise.resolve(parseBody(ctx, function () {
          return Promise.resolve();
        })).then(function ($await_3) {
          try {
            return $return(function () {
              return handler(_objectSpread({}, params, query, ctx.request.body), ctx);
            });
          } catch ($boundEx) {
            return $error($boundEx);
          }
        }, $error);
      });
    });
    return {
      from: from
    };
  },
  middleware: function middleware(deps, service) {
    return function (ctx, next) {
      return new Promise(function ($return, $error) {
        var handler;

        var $Try_1_Post = function () {
          try {
            return $return(next());
          } catch ($boundEx) {
            return $error($boundEx);
          }
        };

        var $Try_1_Catch = function (error) {
          try {
            ctx.body = {
              status: 'failure',
              data: {
                code: error.code,
                message: error.message,
                stack: error.stack,
                meta: error.meta
              }
            };
            return $Try_1_Post();
          } catch ($boundEx) {
            return $error($boundEx);
          }
        };

        try {
          return Promise.resolve(service.from(ctx)).then(function ($await_4) {
            try {
              handler = $await_4;

              if (typeof handler === 'function') {
                return Promise.resolve(handler()).then(function ($await_5) {
                  try {
                    ctx.body = $await_5;
                    return $If_2.call(this);
                  } catch ($boundEx) {
                    return $Try_1_Catch($boundEx);
                  }
                }.bind(this), $Try_1_Catch);
              }

              function $If_2() {
                return $Try_1_Post();
              }

              return $If_2.call(this);
            } catch ($boundEx) {
              return $Try_1_Catch($boundEx);
            }
          }.bind(this), $Try_1_Catch);
        } catch (error) {
          $Try_1_Catch(error);
        }
      });
    };
  }
});

var index = browserPlugin;

export default index;
export { BodyParserOptionsToken, HttpHandlersToken, HttpRouterToken };
//# sourceMappingURL=browser.es5.es.js.map
